;**********************************************************
; PAL Dragon video timing tests
; S. Orchard Oct 2014
;
; In each test, the area highlighted by video
; mode changes should align with the test pattern.
;
; Feel free to use any or all of this source code
;
; Assembled with asm6809 v2.3.1  (www.6809.org.uk)
;
;
;
; v1.2	SO  06.11.14
; Added 2nd 19 phase test (test 9)
;
; The new 19 cycle instruction stream contains an instruction
; which clears the irq flag (clr $ff02).
; This results in no irq during phases 1-6.
;
;
; v1.1	SO
; Added 19 phase irq test (test 8)
;
; The 19 cycle instruction stream contains an instruction
; which clears the irq flag (lda $ff02).
; This results in no irq during phase 6.
;
; Added more comments and flattened 3 and 6 phase test macros
;
;
; v1.0	SO
; Initial version
;
;
; Ideas for future revisions:
; 	Various PAL & NTSC versions
; 	Count hsync interrupts per frame
; 	Check positions of extra PAL lines on Dragon
; 	Fast CPU speed tests (AD and FAST)
; 	Graphics mode changes
; 	SAM registers
; 	CWAI
;**********************************************************

; Macro that implements delay of 2-24 cycles
; Doesn't affect registers, flags or memory
; TODO: Figure out how to do this using elsif
mac_delay_cycles_2_24 macro
	if \1 == 2
		nop
	endif
	if \1 == 3
		orcc #0
	endif
	if \1 == 4
		nop
		nop
	endif
	if \1 == 5
		leau <<0,u
	endif
	if \1 == 6
		tfr a,a
	endif
	if \1 == 7
		leau <<0,u
		nop
	endif
	if \1 == 8
		exg a,a
	endif
	if \1 == 9
		tfr a,a
		orcc #0
	endif
	if \1 == 10
		exg a,a
		nop
	endif
	if \1 == 11
		exg a,a
		orcc #0
	endif
	if \1 == 12
		exg a,a
		nop
		nop
	endif
	if \1 == 13
		exg a,a
		leau <<0,u
	endif
	if \1 == 14
		exg a,a
		tfr a,a
	endif
	if \1 == 15
		exg a,a
		leau <<0,u
		nop
	endif
	if \1 == 16
		exg a,a
		exg a,a
	endif
	if \1 == 17
		exg a,a
		tfr a,a
		orcc #0
	endif
	if \1 == 18
		exg a,a
		exg a,a
		nop
	endif
	if \1 == 19
		exg a,a
		exg a,a
		orcc #0
	endif
	if \1 == 20
		exg a,a
		exg a,a
		nop
		nop
	endif
	if \1 == 21
		exg a,a
		exg a,a
		leau <<0,u
	endif
	if \1 == 22
		exg a,a
		exg a,a
		tfr a,a
	endif
	if \1 == 23
		exg a,a
		exg a,a
		leau <<0,u
		nop
	endif
	if \1 == 24
		exg a,a
		exg a,a
		exg a,a
	endif
	if (\1 < 2) || (\1 > 24)
		tst __bad_parameter_passed_to_delay_cycles_2_24
	endif
	
	endm

; Macro that implements delay of 2 to approx 500000 cycles
; Delay of >24 cycles affects X reg & flags
mac_delay_cycles macro
	if \1 > 24
		ldx #((\1-5) / 8)
90		leax -1,x
		bne 90b
		mac_delay_cycles_2_24 ((\1-5) % 8) + 2
	else
		mac_delay_cycles_2_24 \1
	endif

	endm

;**********************************************************
; Program starts here

irq_vector equ $10d
rst_vector equ $72

	org $4000

start
	nop
	
	ldd #start
	std rst_vector
	
	orcc #$50

1	ldu #test_table
2	ldx ,u++		; get test routine address
	beq 1b			; back to start of table

	pshs x,u

	; wait for keys released
3	jsr check_any_key
	bne 3b

	jsr clear_screen
	ldu #msg_version
	jsr print_text
	ldu #msg_any_key
	jsr print_text
	jsr [,s]		; perform test
	
	puls x,u

	orcc #$50
	jsr disable_fs
	jsr disable_hs

	bra 2b
	
;**********************************************************

test_table
	fdb test_1, test_2
	fdb test_3, test_4, test_5
	fdb test_6_1, test_6_2, test_6_3
	fdb test_7_1, test_7_2, test_7_3
	fdb test_7_4, test_7_5, test_7_6
	fdb test_8_1, test_8_2, test_8_3
	fdb test_8_4, test_8_5, test_8_6
	fdb test_8_7, test_8_8, test_8_9
	fdb test_8_10, test_8_11, test_8_12
	fdb test_8_13, test_8_14, test_8_15
	fdb test_8_16, test_8_17, test_8_18
	fdb test_8_19
	fdb test_9_1,  test_9_2,  test_9_3
	fdb test_9_4,  test_9_5,  test_9_6
	fdb test_9_7,  test_9_8,  test_9_9
	fdb test_9_10, test_9_11, test_9_12
	fdb test_9_13, test_9_14, test_9_15
	fdb test_9_16, test_9_17, test_9_18
	fdb test_9_19
	fdb 0

;**********************************************************

msg_version
	fdb $400+14*32
	fcc "PAL DRAGON TIMING TESTS V1.2",0

msg_any_key
	fdb $400+15*32
	fcc "PRESS ANY KEY",0
	
;**********************************************************
; Macro to produce a mode change over a
; square region of the screen.
; \1 = number of cycles before start of pattern
; \2 = graphics mode for pattern
; \3 = graphics mode for rest of screen

mac_fs_test	macro

5	lda $ff02
	sync

	mac_delay_cycles \1

	ldb #96
2	lda #\2
	sta $ff22
	mac_delay_cycles 5
	lda #\3
	sta $ff22

	; Delay to make each line 57 cycles long
	mac_delay_cycles 33

	decb
	bne 2b
	
	jsr check_any_key
	beq 5b

	endm

;**********************************************************
; Test frame sync falling edge

test_1
	jsr test_pattern
	ldu #msg_1
	jsr print_text

	jsr enable_fs_falling

	mac_fs_test 6851+(48*57), 8, 0
	
	rts

msg_1
	fdb $400
	fcc "1: SYNC ON FS FALLING EDGE",0

;**********************************************************
; Test frame sync rising edge

test_2
	jsr test_pattern
	ldu #msg_2
	jsr print_text

	jsr enable_fs_rising

	mac_fs_test 3602+(48*57), 8, 0
	
	rts

msg_2
	fdb $400
	fcc "2: SYNC ON FS RISING EDGE",0

;**********************************************************
; Test total number of cycles per frame
; Starts from frame sync falling edge then
; continues without any sync.
; PAL Dragon has 312*57 = 17784 cycles per frame

test_3
	jsr test_pattern
	ldu #msg_3
	jsr print_text

	jsr enable_fs_falling

	lda $ff02
	sync

5	mac_delay_cycles 6851+(48*57)
	
	ldb #96

2	lda #8
	sta $ff22
	mac_delay_cycles 5
	lda #0
	sta $ff22
	mac_delay_cycles 33
	decb
	bne 2b
	
	; Delay to make loop exactly one frame
	mac_delay_cycles 47*57+12

	jsr check_any_key	; 29 cycles total
	beq 5b
	
	rts

msg_3
	fdb $400
	fcc "3: FS FALLING EDGE / FREE RUN",0

;**********************************************************
; Macro to produce a mode change on every scanline
; \1 = number of cycles before start of pattern
; \2 = graphics mode for pattern
; \3 = graphics mode for rest of screen

mac_hs_test	macro

5	sync

	mac_delay_cycles \1

	lda #\2
	sta $ff22
	mac_delay_cycles 5
	lda #\3
	sta $ff22
	
	; check for key press
	clra
	sta $ff02
	lda $ff00	; also clears hs flag
	anda #$7f
	cmpa #$7f
	beq 5b

	endm

;**********************************************************
; Timing of horizontal sync falling edge

test_4
	ldu #msg_4
	jsr print_text
	jsr test_pattern_2

	jsr enable_hs_falling

	mac_hs_test 13, 8, 0
	
	rts

msg_4
	fdb $400
	fcc "4: SYNC ON HS FALLING EDGE",0

;**********************************************************
; Timing of horizontal sync rising edge

test_5
	ldu #msg_5
	jsr print_text
	jsr test_pattern_2

	jsr enable_hs_rising

	mac_hs_test 9, 8, 0
	
	rts

msg_5
	fdb $400
	fcc "5: SYNC ON HS RISING EDGE",0

;**********************************************************
; Macro to create a 3 cycle instruction stream that
; can be interrupted in a consistent position
; every frame.
;
; The number of cycles per frame and the total length
; of the irq both need to be divisible by 3
;
; \1	String representing the sub-test number
; \2	Number of cycles to insert between sync
;		and instruction stream.

mac_test_6 macro

test_6_\1
	jsr test_pattern
	ldu #999f
	jsr print_text
	ldx #vsync_handler_6_\1
	stx irq_vector

	jsr enable_fs_falling

	ldx #9f		; address used to break out of loop
	pshs x		; put on stack so irq routine can read it
	
	lda $ff02
	andcc #$ef
	sync		; irq could happen before 1st sync is waiting...
	lda $ff02
	sync		; ...2nd sync is guaranteed to align
	
	; Delay to position frame sync edge at a
	; specific location in instruction stream.
	; A smaller delay causes the edge to appear later
	mac_delay_cycles \2

	; loop until irq routine detects key press
1	bra 1b
	
9	puls x,pc

999	fdb $400
	fcc "6.",\1,": FS IRQ PHASE ",\1,"/3",0

	endm


	mac_test_6 "1", 6
	mac_test_6 "2", 5
	mac_test_6 "3", 4
	
;**********************************************************
; Macro to create a 6 cycle instruction stream that
; can be interrupted in a consistent position
; every frame.
;
; The number of cycles per frame and the total length
; of the irq both need to be divisible by 6
;
; \1	String representing the sub-test number
; \2	Number of cycles to insert between sync
;		and instruction stream.

mac_test_7 macro

test_7_\1
	jsr test_pattern
	ldu #999f
	jsr print_text
	ldx #vsync_handler_7_\1
	stx irq_vector

	jsr enable_fs_falling

	ldx #9f		; address used to break out of loop
	pshs x		; put on stack so irq routine can read it

	ldu #1f		; loop address for jmp <<0,u
	
	lda $ff02
	andcc #$ef
	sync		; irq could happen before 1st sync is waiting...
	lda $ff02
	sync		; ...2nd sync is guaranteed to align
	
	; Delay to position frame sync edge at a
	; specific location in instruction stream.
	; A smaller delay causes the edge to appear later
	mac_delay_cycles \2

	; loop until irq routine detects key press
1	nop
	jmp <<0,u
	
9	puls x,pc

999	fdb $400
	fcc "7.",\1,": FS IRQ PHASE ",\1,"/6",0

	endm


	mac_test_7 "1", 9
	mac_test_7 "2", 8
	mac_test_7 "3", 7
	mac_test_7 "4", 6
	mac_test_7 "5", 5
	mac_test_7 "6", 4


;**********************************************************
; Macro to create a 19 cycle instruction stream
; to be interrupted in a consistent position
; every frame.
;
; The number of cycles per frame and the total length
; of the irq both need to be divisible by 19
;
; \1	String representing the test number
; \2	String representing the sub-test number
;
; \3	Instruction stream variant:
; 	0:  The instruction stream contains lda $ff02 which
;		clears the irq flag.
;		This results in no irq during phase 6.
; 	1:  The instruction stream contains clr $ff02 which
;		clears the irq flag.
;		This results no irq during phases 1-6.
;
; \4	Number of cycles to insert between sync
;		and instruction stream.
;

mac_irq_test macro

test_\1_\2
	jsr test_pattern
	ldu #999f
	jsr print_text
  if (\4 == 30) || ((\3 == 1) && (\4 >= 30))
	ldu #msg_no_irq
	jsr print_text
  endif
	ldx #vsync_handler_\1_\2
	stx irq_vector
	jsr enable_fs_falling

	lda $ff02
	andcc #$ef
	sync		; irq could happen before 1st sync is waiting...
	lda $ff02
	sync		; ...2nd sync is guaranteed to align
	
	; Delay to position frame sync edge at a
	; specific location in instruction stream.
	; A smaller delay causes the edge to appear later
	mac_delay_cycles \4

	; loop until key pressed
  if \3 == 0
1	lda $ff02
	nop
	lda $ff00	; for the key check to work
	anda #$7f	; we're assuming $ff02 has already
	cmpa #$7f	; been written with a zero
	beq 1b		; from a previous check
  endif
  if \3 == 1
1	clr $ff02
	lda $ff00
	anda #$7f
	cmpa #$7f
	beq 1b
  endif
 	
	rts
	
999	fdb $400
	fcc \1,".",\2,": FS IRQ PHASE ",\2,"/19"

  if \3 == 0
	fcc " (A)"
  endif
  if \3 == 1
	fcc " (B)"
  endif

	fcb 0

	endm

msg_no_irq
	fdb $420
	fcc "(IRQ SUPPRESSED - NO ALIGNMENT)",0
	
	mac_irq_test "8", "1", 0, 35
	mac_irq_test "8", "2", 0, 34
	mac_irq_test "8", "3", 0, 33
	mac_irq_test "8", "4", 0, 32
	mac_irq_test "8", "5", 0, 31
	mac_irq_test "8", "6", 0, 30
	mac_irq_test "8", "7", 0, 29
	mac_irq_test "8", "8", 0, 28
	mac_irq_test "8", "9", 0, 27
	mac_irq_test "8", "10", 0, 26
	mac_irq_test "8", "11", 0, 25
	mac_irq_test "8", "12", 0, 24
	mac_irq_test "8", "13", 0, 23
	mac_irq_test "8", "14", 0, 22
	mac_irq_test "8", "15", 0, 21
	mac_irq_test "8", "16", 0, 20
	mac_irq_test "8", "17", 0, 19
	mac_irq_test "8", "18", 0, 18
	mac_irq_test "8", "19", 0, 17

	mac_irq_test "9", "1", 1, 35
	mac_irq_test "9", "2", 1, 34
	mac_irq_test "9", "3", 1, 33
	mac_irq_test "9", "4", 1, 32
	mac_irq_test "9", "5", 1, 31
	mac_irq_test "9", "6", 1, 30
	mac_irq_test "9", "7", 1, 29
	mac_irq_test "9", "8", 1, 28
	mac_irq_test "9", "9", 1, 27
	mac_irq_test "9", "10", 1, 26
	mac_irq_test "9", "11", 1, 25
	mac_irq_test "9", "12", 1, 24
	mac_irq_test "9", "13", 1, 23
	mac_irq_test "9", "14", 1, 22
	mac_irq_test "9", "15", 1, 21
	mac_irq_test "9", "16", 1, 20
	mac_irq_test "9", "17", 1, 19
	mac_irq_test "9", "18", 1, 18
	mac_irq_test "9", "19", 1, 17
	
;**********************************************************
; Macro to create an irq handler with key check
; Modifies return address on stack when key pressed
; \1  Number of cycles to delay before start of pattern
; \2  Number of cycles to delay to make irq specific length

mac_vsync_handler_keychk macro

	; tuning to align pattern
	mac_delay_cycles \1

	ldb #96
2	lda #8
	sta $ff22
	mac_delay_cycles 5
	lda #0
	sta $ff22
	mac_delay_cycles 33
	decb
	bne 2b

	; tuning to make irq handler specific length
	mac_delay_cycles \2

	jsr check_any_key	; 29 cycles total
	beq 1f

	ldx 12,s	; escape from interrupted loop
	stx 10,s
	
1	lda $ff02	; clear vsync irq flag
	rti			; 15 cycles

	endm

;**********************************************************
; Define 3 irq handlers for the 3 phase tests
; Each of these is set up to be a multiple of 6 cycles
; (Though a multiple of 3 would be sufficient)
	
vsync_handler_6_1
	mac_vsync_handler_keychk 6851+(48*57)-23, 7

vsync_handler_6_2
	mac_vsync_handler_keychk 6851+(48*57)-22, 6

vsync_handler_6_3
	mac_vsync_handler_keychk 6851+(48*57)-21, 5

;**********************************************************
; Define 6 irq handlers for the 6 phase tests
; Each of these is set up to be a multiple of 6 cycles
;
; The interrupted instruction stream is the following loop:
;		nop			; 2 cycles
;		jmp <<0,u	; 4 cycles
;
; For phases 1-2 the irq is serviced after nop
; For phases 3-6 the irq is serviced after jmp
	
vsync_handler_7_1
	mac_vsync_handler_keychk 6851+(48*57)-22, 6

vsync_handler_7_2
	mac_vsync_handler_keychk 6851+(48*57)-21, 5

vsync_handler_7_3
	mac_vsync_handler_keychk 6851+(48*57)-24, 8

vsync_handler_7_4
	mac_vsync_handler_keychk 6851+(48*57)-23, 7

vsync_handler_7_5
	mac_vsync_handler_keychk 6851+(48*57)-22, 6

vsync_handler_7_6
	mac_vsync_handler_keychk 6851+(48*57)-21, 5

;**********************************************************
; Macro to create an irq handler without key check
; \1  Number of cycles to delay before start of pattern
; \2  Number of cycles to delay to make irq specific length

mac_vsync_handler macro

	; tuning to align pattern
	mac_delay_cycles \1

	ldb #96
2	lda #8
	sta $ff22
	mac_delay_cycles 5
	lda #0
	sta $ff22
	mac_delay_cycles 33
	decb
	bne 2b

	; tuning to make irq handler specific length
	mac_delay_cycles \2
	
	lda $ff02	; clear vsync irq flag
	rti			; 15 cycles

	endm

;**********************************************************
; Define 19 irq handlers for the A set of 19 phase tests
; Each of these is set up to be a multiple of 19 cycles
;
; The interrupted instruction stream is the following loop:
; loop	lda $ff02	; 5 cycles
;		nop			; 2 cycles
;		lda $ff00	; 5 cycles
;		anda #$7f	; 2 cycles
;		cmpa #$7f	; 2 cycles
;		beq loop	; 3 cycles
;
; For phases 1-5 the irq is serviced after lda $ff02
; For phases 6-7 the irq should be serviced after nop
; but is suppressed during phase 6.

vsync_handler_8_1
	mac_vsync_handler 6851+(48*57)-25, 7

vsync_handler_8_2
	mac_vsync_handler 6851+(48*57)-24, 6

vsync_handler_8_3
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_8_4
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_5
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_8_6
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_7
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_8_8
	mac_vsync_handler 6851+(48*57)-25, 7

vsync_handler_8_9
	mac_vsync_handler 6851+(48*57)-24, 6

vsync_handler_8_10
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_8_11
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_12
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_8_13
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_14
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_8_15
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_16
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_8_17
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_8_18
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_8_19
	mac_vsync_handler 6851+(48*57)-21, 3

;**********************************************************
; Define 19 irq handlers for the B set of 19 phase tests
; Each of these is set up to be a multiple of 19 cycles
;
; The interrupted instruction stream is the following loop:
; loop	clr $ff02	; 7 cycles
;		lda $ff00	; 5 cycles
;		anda #$7f	; 2 cycles
;		cmpa #$7f	; 2 cycles
;		beq loop	; 3 cycles
;
; For phases 1-7 the irq should be serviced after clr $ff02
; but is suppressed during phases 1-6

vsync_handler_9_1
	mac_vsync_handler 6851+(48*57)-27, 9

vsync_handler_9_2
	mac_vsync_handler 6851+(48*57)-26, 8

vsync_handler_9_3
	mac_vsync_handler 6851+(48*57)-25, 7

vsync_handler_9_4
	mac_vsync_handler 6851+(48*57)-24, 6

vsync_handler_9_5
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_9_6
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_9_7
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_9_8
	mac_vsync_handler 6851+(48*57)-25, 7

vsync_handler_9_9
	mac_vsync_handler 6851+(48*57)-24, 6

vsync_handler_9_10
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_9_11
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_9_12
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_9_13
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_9_14
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_9_15
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_9_16
	mac_vsync_handler 6851+(48*57)-21, 3

vsync_handler_9_17
	mac_vsync_handler 6851+(48*57)-23, 5

vsync_handler_9_18
	mac_vsync_handler 6851+(48*57)-22, 4

vsync_handler_9_19
	mac_vsync_handler 6851+(48*57)-21, 3

;**********************************************************

; Test pattern for vertical timing
; Square checkerboard in centre of screen

test_pattern
	lda #8
	sta ,-s
	ldd #$2060
	ldx #1024+4*32+10
1	std ,x
	std 2,x
	std 4,x
	std 6,x
	std 8,x
	std 10,x
	leax 32,x
	exg a,b
	dec ,s
	bne 1b
	puls a,pc

; Test pattern for horizontal timing
; Vertical checkered stripe down centre of screen

test_pattern_2
	ldx #16
	ldu #1024+10
1	ldb #6
2	lda ,u
	eora #$40
	sta ,u++
	decb
	bne 2b
	leau 21,u
	ldb #6
2	lda ,u
	eora #$40
	sta ,u++
	decb
	bne 2b
	leau 19,u
	leax -1,x
	bne 1b
	rts


clear_screen
	ldx #1024
	ldd #$6060
1	std ,x++
	cmpx #1536
	blo 1b
	rts


check_any_key		; 21 cycles (not including call)
	clra
	sta $ff02
	lda $ff00
	anda #$7f
	cmpa #$7f
	rts


print_text
	ldx ,u++
1	lda ,u+
	beq 9f
	cmpa #'A
	bhs 2f
	eora #$40
2	sta ,x+
	bra 1b
9	rts

;**********************************************************

; enable vsync irq on falling edge
enable_fs_falling
	lda $ff03
	anda #$fd
	ora #1
	sta $ff03
	rts

; enable vsync irq on rising edge
enable_fs_rising
	lda $ff03
	ora #3
	sta $ff03
	rts

; disable vsync irq
disable_fs
	lda $ff03
	anda #$fe
	sta $ff03
	rts

; enable hsync irq on falling edge
enable_hs_falling
	lda $ff01
	anda #$fd
	ora #1
	sta $ff01
	rts

; enable hsync irq on rising edge
enable_hs_rising
	lda $ff01
	ora #3
	sta $ff01
	rts

; disable hsync irq
disable_hs
	lda $ff01
	anda #$fe
	sta $ff01
	rts

